// ---------------------------------------------------------------------
//
// JavaBridge
//
// Copyright (c) 2012-2020 WINSOFT
//
// ---------------------------------------------------------------------

unit JavaBridge;

//{$DEFINE TRIAL} // trial version, comment this line for full version

{$ifndef CONDITIONALEXPRESSIONS}
  {$define MSWINDOWS}
{$endif CONDITIONALEXPRESSIONS}

{$ifdef FPC}
  {$define MSWINDOWS}
{$endif FPC}

interface

uses jni {$ifdef MSWINDOWS} , Windows {$endif MSWINDOWS};

{$ifdef MSWINDOWS}
var
  JvmLibraryName: string = '';
  JvmLibrary: HMODULE;
  JNI_GetDefaultJavaVMInitArgs: TJNI_GetDefaultJavaVMInitArgs;
  JNI_CreateJavaVM: TJNI_CreateJavaVM;
  JNI_GetCreatedJavaVMs: TJNI_GetCreatedJavaVMs;

function Loaded: Boolean;
procedure LoadLibrary;
procedure UnloadLibrary;
{$endif MSWINDOWS}

{$ifdef MACOS}
const
  libJavaVM = '/System/Library/Frameworks/JavaVM.framework/JavaVM';

function JNI_GetDefaultJavaVMInitArgs(out Args: JavaVMInitArgs): JInt; cdecl; external libJavaVM name '_JNI_GetDefaultJavaVMInitArgs';
function JNI_CreateJavaVM(out VM: Pointer { JavaVM }; out Env: Pointer { JNIEnv }; var Args: JavaVMInitArgs): JInt; cdecl; external libJavaVM name '_JNI_CreateJavaVM';
function JNI_GetCreatedJavaVMs(out PVM: Pointer { JavaVM }; Size: JSize; out OutSize: JSize): JInt; cdecl; external libJavaVM name '_JNI_GetCreatedJavaVMs';
{$endif MACOS}

// Utilities

function CreateJString(Env: Pointer; Text: PAnsiChar): JString; overload;
function CreateJString(Env: Pointer; Text: PWideChar): JString; overload;
function CreateJString(Env: Pointer; const Text: AnsiString): JString; overload;
function CreateJString(Env: Pointer; const Text: WideString): JString; overload;

type
  TJValueDynArray = array of JValue;

function Args(Env: Pointer; const Args: array of const): TJValueDynArray;

implementation

uses {$ifdef MSWINDOWS} Registry, {$endif MSWINDOWS} SysUtils {$ifndef FPC} {$ifdef TRIAL} {$ifdef MACOS} {$else} , Forms {$endif MACOS} {$endif TRIAL} {$endif FPC};

function CreateJString(Env: Pointer; Text: PAnsiChar): JString; overload;
begin
  Result := JNIEnv(Env).NewStringUTF(Text);
end;

function CreateJString(Env: Pointer; Text: PWideChar): JString; overload;
begin
  Result := JNIEnv(Env).NewString(PJChar(Text), Length(Text));
end;

function CreateJString(Env: Pointer; const Text: AnsiString): JString; overload;
begin
  Result := CreateJString(Env, PAnsiChar(Text));
end;

function CreateJString(Env: Pointer; const Text: WideString): JString; overload;
begin
  Result := CreateJString(Env, PWideChar(Text));
end;

function Args(Env: Pointer; const Args: array of const): TJValueDynArray;
var
  I: Integer;
begin
  SetLength(Result, Length(Args));
  for I := Low(Args) to High(Args) do
    case Args[I].VType of
      vtInteger:
        Result[I].I := Args[I].VInteger;
      vtBoolean:
        Result[I].Z := Args[I].VBoolean;
      vtChar:
        Result[I].C := WideChar(Args[I].VChar);
      vtExtended:
        Result[I].D := Args[I].VExtended^;
      vtPointer:
        Result[I].L := JObject(Args[I].VPointer);
      vtWideChar:
        Result[I].C := Args[I].VWideChar;
      vtAnsiString:
        Result[I].L := CreateJString(Env, PAnsiChar(Args[I].VAnsiString));
      vtWideString:
        Result[I].L := CreateJString(Env, PWideChar(Args[I].VWideString));
      vtInt64:
        Result[I].J := Args[I].VInt64^;
    else
      raise Exception.Create('Unsupported argument type');
    end;
end;

{$ifdef MSWINDOWS}

function Loaded: Boolean;
begin
  Result := JvmLibrary <> 0;
end;

procedure UnloadLibrary;
begin
  if Loaded then
  begin
    FreeLibrary(JvmLibrary);
    JvmLibrary := 0;
  end;
end;

procedure CheckLoadLibrary(const Name: string);
begin
  JvmLibrary := Windows.LoadLibrary(PChar(Name));
  if JvmLibrary = 0 then
    raise Exception.Create(SysErrorMessage(GetLastError) + ': ' + Name);
end;

function CheckGetProcAddress(const Name: string): Pointer;
var
  ErrorMessage: string;
begin
  Result := GetProcAddress(JvmLibrary, PChar(Name));
  if Result = nil then
  begin
    ErrorMessage := SysErrorMessage(GetLastError) + ': ' + Name;
    UnloadLibrary;
    raise Exception.Create(ErrorMessage);
  end;
end;

function GetDllName: string;
var
  Registry: TRegistry;
  Version: string;
begin
  Result := 'jvm.dll';

  Registry := TRegistry.Create(KEY_READ);
  try
    Registry.RootKey := HKEY_LOCAL_MACHINE;
    if Registry.OpenKey('\SOFTWARE\JavaSoft\Java Runtime Environment', False) then
    begin
      Version := Registry.ReadString('CurrentVersion');
      if Version <> '' then
        if Registry.OpenKey(Version, False) then
          Result := Registry.ReadString('RuntimeLib');
    end;
  finally
    Registry.Free;
  end;
end;

procedure LoadLibrary;
begin
  if not Loaded then
  begin
    if JvmLibraryName = '' then
      JvmLibraryName := GetDllName;
    CheckLoadLibrary(JvmLibraryName);
    JNI_GetDefaultJavaVMInitArgs := CheckGetProcAddress('JNI_GetDefaultJavaVMInitArgs');
    JNI_CreateJavaVM := CheckGetProcAddress('JNI_CreateJavaVM');
    JNI_GetCreatedJavaVMs := CheckGetProcAddress('JNI_GetCreatedJavaVMs');
  end;
end;
{$endif MSWINDOWS}

initialization

{$IFDEF TRIAL}

{$ifdef MACOS}
  WriteLn('JavaBridge, Copyright (c) 2012-2020 WINSOFT');
  WriteLn;
  WriteLn('A trial version of JavaBridge started.');
  WriteLn;
  WriteLn('Please note that trial version is supposed to be used for evaluation only.');
  WriteLn('If you wish to distribute JavaBridge as part of your application,');
  WriteLn('you must register from website at https://www.winsoft.sk.');
  WriteLn;
  WriteLn('Thank you for trialing JavaBridge.');
  WriteLn;
  WriteLn('Press Enter key to continue');
  ReadLn;
{$else}
  MessageBox({$ifdef FPC} 0 {$else} Application.Handle {$endif FPC},
    'A trial version of JavaBridge started.' + #13#13 +
    'Please note that trial version is supposed to be used for evaluation only. ' +
    'If you wish to distribute JavaBridge as part of your application, ' +
    'you must register from website at https://www.winsoft.sk.' + #13#13 +
    'Thank you for trialing JavaBridge.',
    'JavaBridge, Copyright (c) 2012-2020 WINSOFT',
     MB_OK or MB_ICONINFORMATION);
{$endif MACOS}

{$ENDIF TRIAL}

end.